import os
from typing import Optional

import inquirer
import typer

from savvihub.api.exceptions import NotFoundAPIException
from savvihub.api.uploader import Uploader
from savvihub.api.zipper import Zipper
from savvihub.cli.commands.volume import parse_remote_volume_path
from savvihub.cli.constants import PROJECT_TYPE_CLI_DRIVEN, PROJECT_TYPE_VERSION_CONTROL
from savvihub.cli.exceptions import ExitException
from savvihub.cli.typer import Context


def message_callback(ctx: Context, message: str) -> str:
    if message:
        return message

    inquirer_message = ""
    if ctx.project.type == PROJECT_TYPE_VERSION_CONTROL:
        inquirer_message = "Experiment message(set to commit message if you passed empty)"
    elif ctx.project.type == PROJECT_TYPE_CLI_DRIVEN:
        inquirer_message = "Experiment message"

    return inquirer.prompt([inquirer.Text(
        'message',
        message=inquirer_message,
        default=""
    )], raise_keyboard_interrupt=True).get('message')




# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import inspect
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ModelBillingHistory(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'amount': 'float',
        'billing_history_organization': 'int',
        'billing_history_workload': 'int',
        'created_dt': 'datetime',
        'edges': 'ModelBillingHistoryEdges',
        'id': 'int',
        'month': 'datetime',
        'total': 'float',
        'updated_dt': 'datetime',
        'workload_created_dt': 'datetime'
    }

    attribute_map = {
        'amount': 'amount',
        'billing_history_organization': 'billing_history_organization',
        'billing_history_workload': 'billing_history_workload',
        'created_dt': 'created_dt',
        'edges': 'edges',
        'id': 'id',
        'month': 'month',
        'total': 'total',
        'updated_dt': 'updated_dt',
        'workload_created_dt': 'workload_created_dt'
    }

    def __init__(self, amount=None, billing_history_organization=None, billing_history_workload=None, created_dt=None, edges=None, id=None, month=None, total=None, updated_dt=None, workload_created_dt=None, local_vars_configuration=None):  # noqa: E501
        """ModelBillingHistory - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._amount = None
        self._billing_history_organization = None
        self._billing_history_workload = None
        self._created_dt = None
        self._edges = None
        self._id = None
        self._month = None
        self._total = None
        self._updated_dt = None
        self._workload_created_dt = None
        self.discriminator = None

        if amount is not None:
            self.amount = amount
        if billing_history_organization is not None:
            self.billing_history_organization = billing_history_organization
        if billing_history_workload is not None:
            self.billing_history_workload = billing_history_workload
        self.created_dt = created_dt
        if edges is not None:
            self.edges = edges
        if id is not None:
            self.id = id
        if month is not None:
            self.month = month
        if total is not None:
            self.total = total
        self.updated_dt = updated_dt
        if workload_created_dt is not None:
            self.workload_created_dt = workload_created_dt

    @property
    def amount(self):
        """Gets the amount of this ModelBillingHistory.  # noqa: E501


        :return: The amount of this ModelBillingHistory.  # noqa: E501
        :rtype: float
        """
        return self._amount

    @amount.setter
    def amount(self, amount):
        """Sets the amount of this ModelBillingHistory.


        :param amount: The amount of this ModelBillingHistory.  # noqa: E501
        :type amount: float
        """

        self._amount = amount

    @property
    def billing_history_organization(self):
        """Gets the billing_history_organization of this ModelBillingHistory.  # noqa: E501


        :return: The billing_history_organization of this ModelBillingHistory.  # noqa: E501
        :rtype: int
        """
        return self._billing_history_organization

    @billing_history_organization.setter
    def billing_history_organization(self, billing_history_organization):
        """Sets the billing_history_organization of this ModelBillingHistory.


        :param billing_history_organization: The billing_history_organization of this ModelBillingHistory.  # noqa: E501
        :type billing_history_organization: int
        """

        self._billing_history_organization = billing_history_organization

    @property
    def billing_history_workload(self):
        """Gets the billing_history_workload of this ModelBillingHistory.  # noqa: E501


        :return: The billing_history_workload of this ModelBillingHistory.  # noqa: E501
        :rtype: int
        """
        return self._billing_history_workload

    @billing_history_workload.setter
    def billing_history_workload(self, billing_history_workload):
        """Sets the billing_history_workload of this ModelBillingHistory.


        :param billing_history_workload: The billing_history_workload of this ModelBillingHistory.  # noqa: E501
        :type billing_history_workload: int
        """

        self._billing_history_workload = billing_history_workload

    @property
    def created_dt(self):
        """Gets the created_dt of this ModelBillingHistory.  # noqa: E501


        :return: The created_dt of this ModelBillingHistory.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ModelBillingHistory.


        :param created_dt: The created_dt of this ModelBillingHistory.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def edges(self):
        """Gets the edges of this ModelBillingHistory.  # noqa: E501


        :return: The edges of this ModelBillingHistory.  # noqa: E501
        :rtype: ModelBillingHistoryEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this ModelBillingHistory.


        :param edges: The edges of this ModelBillingHistory.  # noqa: E501
        :type edges: ModelBillingHistoryEdges
        """

        self._edges = edges

    @property
    def id(self):
        """Gets the id of this ModelBillingHistory.  # noqa: E501


        :return: The id of this ModelBillingHistory.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ModelBillingHistory.


        :param id: The id of this ModelBillingHistory.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def month(self):
        """Gets the month of this ModelBillingHistory.  # noqa: E501


        :return: The month of this ModelBillingHistory.  # noqa: E501
        :rtype: datetime
        """
        return self._month

    @month.setter
    def month(self, month):
        """Sets the month of this ModelBillingHistory.


        :param month: The month of this ModelBillingHistory.  # noqa: E501
        :type month: datetime
        """

        self._month = month

    @property
    def total(self):
        """Gets the total of this ModelBillingHistory.  # noqa: E501


        :return: The total of this ModelBillingHistory.  # noqa: E501
        :rtype: float
        """
        return self._total

    @total.setter
    def total(self, total):
        """Sets the total of this ModelBillingHistory.


        :param total: The total of this ModelBillingHistory.  # noqa: E501
        :type total: float
        """

        self._total = total

    @property
    def updated_dt(self):
        """Gets the updated_dt of this ModelBillingHistory.  # noqa: E501


        :return: The updated_dt of this ModelBillingHistory.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this ModelBillingHistory.


        :param updated_dt: The updated_dt of this ModelBillingHistory.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    @property
    def workload_created_dt(self):
        """Gets the workload_created_dt of this ModelBillingHistory.  # noqa: E501


        :return: The workload_created_dt of this ModelBillingHistory.  # noqa: E501
        :rtype: datetime
        """
        return self._workload_created_dt

    @workload_created_dt.setter
    def workload_created_dt(self, workload_created_dt):
        """Sets the workload_created_dt of this ModelBillingHistory.


        :param workload_created_dt: The workload_created_dt of this ModelBillingHistory.  # noqa: E501
        :type workload_created_dt: datetime
        """

        self._workload_created_dt = workload_created_dt

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = inspect.getargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ModelBillingHistory):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ModelBillingHistory):
            return True

        return self.to_dict() != other.to_dict()



import tempfile
from typing import Optional, Union, Iterable, Text, IO, Any, Mapping
from unittest import TestCase

from click.testing import Result
from typer.testing import CliRunner as TyperCliRunner

from savvihub.api.savvihub import SavviHubClient
from savvihub.cli.commands.main import app
from savvihub.cli.git import GitRepository
from savvihub.cli.typer import Context
from savvihub.common import constants
from savvihub.common.utils import random_string


class CliRunner(TyperCliRunner):
    def invoke(
        self,
        args: Optional[Union[str, Iterable[str]]] = None,
        input: Optional[Union[bytes, Text, IO[Any]]] = None,
        env: Optional[Mapping[str, str]] = None,
        color: bool = False,
        allow_fail: bool = False,
        **extra: Any,
    ) -> Result:
        result = super().invoke(app, args, input, env, catch_exceptions=False, color=color, **extra)
        if not allow_fail:
            assert result.exit_code == 0, result.output
        return result


class BaseTestCase(TestCase):
    def setUp(self):
        super().setUp()
        constants.TEST = True
        self._auto_savvi_init()
        self._monkey_patch(self.token, self.organization_name)
        self.runner = CliRunner()

    def _auto_savvi_init(self):
        # user init
        username = random_string()
        unauthorized_client = SavviHubClient()
        cli_token = unauthorized_client.signin_cli_token().cli_token
        jwt_token = unauthorized_client.signup_for_test_only(
            email=f'{username}@savvihub.test',
            username=username,
            name=username,
            password='testtest',
            invitation_token='invitation_token_for_cli_test',
        ).token

        jwt_client = SavviHubClient(auth_header={'Authorization': f'JWT {jwt_token}'})
        jwt_client.signin_confirm_for_test_only(cli_token=cli_token)

        check_signin_response = jwt_client.check_signin(cli_token)
        if not check_signin_response.signin_success:
            raise Exception('Signin Failed')

        # project init
        access_token_client = SavviHubClient(auth_header={'Authorization': f'Token {check_signin_response.access_token}'})
        organization_name = access_token_client.organization_list().organizations[0].name

        self.token = check_signin_response.access_token
        self.organization_name = organization_name

    @staticmethod
    def _monkey_patch(token, organization_name):
        class MockConfig:
            def __init__(self):
                self.token = token
                self.organization = organization_name

        def context_load(
            self,
            auth_required=False,
            user_required=False,
        ):
            mock_config = MockConfig()
            self.global_config = mock_config
            if auth_required:
                self.authenticated_client.verify_access_token()

            if user_required:
                self.user = self.authenticated_client.get_my_info()

        Context.load = context_load

        class MockGitRepo:
            @classmethod
            def enable(cls):
                GitRepository.check_revision_in_remote = cls.check_revision_in_remote
                GitRepository.get_commit_message = cls.get_commit_message
                GitRepository.get_current_diff_status = cls.get_current_diff_status
                GitRepository.get_current_diff_file = cls.get_current_diff_file
                GitRepository._get_remote_revision = cls.get_remote_revision

            def check_revision_in_remote(self, ref):
                return True

            def get_remote_revision(self):
                return 'dummy-ref', 'dummy-branch', False

            def get_commit_message(self, ref):
                return 'dummy-commit-message'

            def get_current_diff_status(self, ref):
                return True, {
                    'untracked': ['dummy-untracked-file.py', 'dummy-untracked-file.go'],
                    'uncommitted': ['dummy-uncommitted-file.py', 'dummy-uncommitted-file.go'],
                }

            def get_current_diff_file(self, git_ref, with_untracked=True):
                fp = tempfile.NamedTemporaryFile(suffix='.patch')
                fp.write('dummy-diff'.encode())
                return fp

        MockGitRepo.enable()



from typing import List

from openapi_client import ProtoVolumeMountRequest, ProtoVolumeMountRequestSourceDataset, \
    ProtoVolumeMountRequestSourceVolume
from savvihub.api.exceptions import NotFoundAPIException
from savvihub.cli.exceptions import ExitException
from savvihub.cli.typer import Context
from savvihub.cli.utils import parse_dataset


def dataset_mount_callback(ctx: Context, dataset_mounts: List[str]) -> List[str]:
    client = ctx.authenticated_client
    organization_name = ctx.params['organization_name']
    ctx.store['dataset_mounts'] = []
    for dataset_mount in dataset_mounts:
        splitted = dataset_mount.split(':')
        if len(splitted) != 2:
            raise ExitException(f'Invalid dataset mount format: {dataset_mount}. '
                                f'You should specify both mount path and dataset name.\n'
                                f'ex) /input/dataset1:mnist@3d1e0f91c')

        mount_path, dataset_full_name = splitted
        organization_name_override, dataset_name, dataset_version_name = parse_dataset(dataset_full_name)
        if organization_name_override:
            organization_name = organization_name_override

        dataset = client.dataset_read(organization_name, dataset_name)
        if dataset_version_name != 'latest':
            try:
                client.dataset_version_read(dataset.volume_id, dataset_version_name)
            except NotFoundAPIException:
                raise ExitException(f'Invalid dataset dataset_version: {dataset_full_name}\n'
                                    f'Please check your dataset and dataset_version exist in organization `{organization_name}`.')

        ctx.store['dataset_mounts'].append(ProtoVolumeMountRequest(
            mount_type='dataset',
            mount_path=mount_path,
            dataset=ProtoVolumeMountRequestSourceDataset(
                dataset_id=dataset.id,
                dataset_version_name=dataset_version_name,
            ),
        ))

    return dataset_mounts


def volume_file_mount_callback(ctx: Context, volume_file_mounts: List[str]) -> List[str]:
    ctx.store['volume_file_mounts'] = []
    for volume_file_mount in volume_file_mounts:
        splitted = volume_file_mount.split(':')
        if len(splitted) != 2:
            raise ExitException(f'Invalid volume file mount format: {volume_file_mount}. '
                                f'You should specify both mount path and volume file.\n'
                                f'Volume file can be notated with volume id and subpath.'
                                f'ex) /input/model.pt:1#subPath=f72fca375e812/model.pt')

        mount_path, volume_id = splitted
        if '#subPath=' in volume_id:
            volume_id, sub_path = volume_id.split('#subPath=')
        else:
            sub_path = ''

        try:
            volume_id = int(volume_id)
        except ValueError:
            raise ExitException(f'Volume id should be an integer.')

        try:
            ctx.authenticated_client.volume_read(volume_id)
        except NotFoundAPIException:
            raise ExitException(f'Volume not found: {volume_id}')

        ctx.store['volume_file_mounts'].append(ProtoVolumeMountRequest(
            mount_type='volume',
            mount_path=mount_path,
            volume=ProtoVolumeMountRequestSourceVolume(
                volume_id=volume_id,
                sub_path=sub_path,
            )
        ))

    return volume_file_mounts



import os
from typing import Optional

import yaml

from savvihub.cli.exceptions import ExitException
from savvihub.cli.typer import Context


def sweep_config_file_callback(ctx: Context, filepath: str) -> Optional[str]:
    if not filepath:
        return

    if not os.path.isfile(filepath):
        raise ExitException(f'File does not exists: {filepath}')

    try:
        configs = yaml.safe_load(filepath)
    except yaml.YAMLError:
        raise ExitException(f'Invalid YAML: {filepath}')

    with open(filepath, 'r') as stream:
        configs = yaml.load(stream, Loader=yaml.FullLoader)

    ctx.store['objective'] = configs['objective']
    ctx.store['algorithm'] = configs['algorithm']
    ctx.store['search_space'] = configs['search_space']['parameters']
    ctx.store['parallel_experiment_count'] = configs['parallel_experiment_count']
    ctx.store['max_experiment_count'] = configs['max_experiment_count']
    ctx.store['max_failed_experiment_count'] = configs['max_failed_experiment_count']
    return



import concurrent.futures
import json
import os
import random
import shutil
import string
import uuid
import zlib
from datetime import datetime, timedelta, timezone

import timeago


def make_file(path):
    os.makedirs(os.path.dirname(path), exist_ok=True)
    if not os.path.exists(path):
        with open(path, 'w'):
            print(" [*] Make a file : {}".format(path))
            pass


def make_dir(path):
    if not os.path.exists(path):
        print(" [*] Make directories : {}".format(path))
        os.makedirs(path)


def remove_file(path):
    if os.path.exists(path):
        print(" [*] Removed: {}".format(path))
        os.remove(path)


def remove_dir(path):
    if os.path.exists(path):
        print(" [*] Removed: {}".format(path))
        shutil.rmtree(path)


def write_json(path, data):
    with open(path, 'w', encoding='utf-8') as f:
        json.dump(data, f, indent=4, sort_keys=True, ensure_ascii=False)


def load_json(path):
    with open(path) as f:
        data = json.loads(f.read())
    return data


def calculate_crc32c(filename):
    try:
        with open(filename, 'rb') as fh:
            h = 0
            while True:
                s = fh.read(65536)
                if not s:
                    break
                h = zlib.crc32(s, h)
            return "%X" % (h & 0xFFFFFFFF)
    except Exception as e:
        import sentry_sdk
        sentry_sdk.capture_exception(e)
        sentry_sdk.flush()

        return ""


def read_in_chunks(filename, blocksize=65535, chunks=-1, callback=None):
    """Lazy function (generator) to read a file piece by piece."""
    with open(filename, 'rb') as f:
        while chunks:
            data = f.read(blocksize)
            if not data:
                break
            yield data

            if callback:
                callback(data)
            chunks -= 1


def wait_all_futures(futures):
    concurrent.futures.wait(futures)
    return [future.result() for future in futures]


def short_string(data, length):
    return (data[:length] + '..') if len(data) > length else data


def parse_str_time_to_datetime(time_str):
    return datetime.strptime(time_str, "%Y-%m-%dT%H:%M:%S.%fZ")


def parse_time_to_ago(dt: datetime):
    if not dt:
        return 'N/A'
    return timeago.format(dt, datetime.now(timezone.utc))


def parse_timestamp_or_none(timestamp):
    if not timestamp:
        return 'N/A'
    return datetime.fromtimestamp(timestamp)


def sizeof_fmt(num, suffix='B'):
    for unit in ['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z']:
        if abs(num) < 1024.0:
            return "%3.1f%s%s" % (num, unit, suffix)
        num /= 1024.0
    return "%.1f%s%s" % (num, 'Yi', suffix)


def remove_prefix(self: str, prefix: str) -> str:
    if self.startswith(prefix):
        return self[len(prefix):]
    else:
        return self[:]


def remove_suffix(self: str, suffix: str) -> str:
    # suffix='' should not call self[:-0].
    if suffix and self.endswith(suffix):
        return self[:-len(suffix)]
    else:
        return self[:]


def random_string(length=24, punctuations=False):
    letters = string.ascii_lowercase + string.digits
    if punctuations:
        letters += '!@#$^&'
    return random.SystemRandom().choice(string.ascii_lowercase) + ''.join(random.SystemRandom().choice(letters) for _ in range(length-1))


def random_uuid():
    return str(uuid.uuid4())


def random_date(start, end):
    delta = end - start
    random_days = random.randrange(delta.days)
    return start + timedelta(days=random_days)



import typer

from savvihub.api.exceptions import NotFoundAPIException
from savvihub.cli.commands.volume import volume_file_list, volume_file_remove, volume_file_copy
from savvihub.cli.exceptions import ExitException
from savvihub.cli.inputs.organization import organization_name_option
from savvihub.cli.typer import Typer, Context
from savvihub.cli.utils import parse_dataset

dataset_files_app = Typer()


@dataset_files_app.callback()
def main():
    """
    Manage files in the dataset
    """
    return


@dataset_files_app.command(user_required=True)
def ls(
    ctx: Context,
    dataset_full_name: str = typer.Argument(..., metavar='DATASET',
                                            help='{organization}/{dataset}@{dataset_version} format '
                                                 'where organization and dataset_version are optional.'),
    path: str = typer.Argument('/'),
    organization_name: str = organization_name_option,
    recursive: bool = typer.Option(False, '-r', '--recursive', help='recursive flag'),
    directory: bool = typer.Option(False, '-d', '--directory', help='list the directory itself, not its contents')
):
    """
    List files in the dataset with prefix
    """
    organization_name_override, dataset_name, dataset_version_name = parse_dataset(dataset_full_name)
    if organization_name_override:
        organization_name = organization_name_override

    dataset = ctx.authenticated_client.dataset_read(organization_name, dataset_name)
    volume_file_list(ctx, dataset.volume_id, dataset_version_name, path, recursive, directory)


@dataset_files_app.command(user_required=True)
def rm(
    ctx: Context,
    dataset_full_name: str = typer.Argument(..., metavar='DATASET_NAME'),
    path: str = typer.Argument(...),
    organization_name: str = organization_name_option,
    recursive: bool = typer.Option(False, '-r', '-R', '--recursive',
                                   help='Remove directories and their contents recursively'),
):
    """
    Remove files in a dataset (SavviHub dataset files only)
    """
    organization_name_override, dataset_name, dataset_version_name = parse_dataset(dataset_full_name)
    if organization_name_override:
        organization_name = organization_name_override

    if dataset_version_name != 'latest':
        raise ExitException('Files in a dataset_version cannot be removed.')

    dataset = ctx.authenticated_client.dataset_read(organization_name, dataset_name)
    volume_file_remove(ctx, dataset.volume_id, path, recursive)


@dataset_files_app.command(user_required=True)
def upload(
    ctx: Context,
    dataset_full_name: str = typer.Argument(..., metavar='DATASET_NAME'),
    source_path: str = typer.Argument(...),
    dest_path: str = typer.Argument(...),
    organization_name: str = organization_name_option,
    recursive: bool = typer.Option(False, '-r', '--recursive'),
    watch: bool = typer.Option(False, '-w', '--watch'),
):
    """
    Upload files to a dataset (SavviHub dataset only)
    """
    organization_name_override, dataset_name, dataset_version_name = parse_dataset(dataset_full_name)
    if organization_name_override:
        organization_name = organization_name_override

    if dataset_version_name != 'latest':
        raise ExitException('Files in a dataset_version cannot be removed.')

    dataset = ctx.authenticated_client.dataset_read(organization_name, dataset_name)
    volume_file_copy(
        ctx,
        source_volume_id=None,
        source_dataset_version=None,
        source_path=source_path,
        dest_volume_id=dataset.volume_id,
        dest_dataset_version='latest',
        dest_path=dest_path,
        recursive=recursive,
        watch=watch,
    )


@dataset_files_app.command(user_required=True)
def download(
    ctx: Context,
    dataset_full_name: str = typer.Argument(..., metavar='DATASET',
                                            help='{organization}/{dataset}@{dataset_version} format '
                                                 'where organization and dataset_version are optional.'),
    source_path: str = typer.Argument(...),
    dest_path: str = typer.Argument('.'),
    organization_name: str = organization_name_option,
    recursive: bool = typer.Option(False, '-r', '--recursive'),
    watch: bool = typer.Option(False, '-w', '--watch'),
):
    """
    Download files from a dataset
    """
    organization_name_override, dataset_name, dataset_version_name = parse_dataset(dataset_full_name)
    if organization_name_override:
        organization_name = organization_name_override

    dataset = ctx.authenticated_client.dataset_read(organization_name, dataset_name)
    if dataset_version_name != 'latest':
        try:
            ctx.authenticated_client.dataset_version_read(dataset.volume_id, dataset_version_name)
        except NotFoundAPIException:
            raise ExitException(f'Invalid dataset dataset_version: {dataset_full_name}\n'
                                f'Please check your dataset and dataset_version exist in organization `{organization_name}`.')
    volume_file_copy(
        ctx,
        source_volume_id=dataset.volume_id,
        source_dataset_version=dataset_version_name,
        source_path=source_path,
        dest_volume_id=None,
        dest_dataset_version=None,
        dest_path=dest_path,
        recursive=recursive,
        watch=watch,
    )


@dataset_files_app.command(user_required=True)
def cp(
    ctx: Context,
    dataset_full_name: str = typer.Argument(..., metavar='DATASET',
                                            help='{organization}/{dataset}@{dataset_version} format '
                                                 'where organization and dataset_version are optional. '
                                                 'If a dataset_version is specified, '
                                                 'copies the files from the dataset_version to the current dataset.'),
    source_path: str = typer.Argument(...),
    dest_path: str = typer.Argument(...),
    organization_name: str = organization_name_option,
    recursive: bool = typer.Option(False, '-r', '--recursive'),
    watch: bool = typer.Option(False, '-w', '--watch'),
):
    """
    Copy files within a dataset (SavviHub dataset files only)
    """
    organization_name_override, dataset_name, src_dataset_version_name = parse_dataset(dataset_full_name)
    if organization_name_override:
        organization_name = organization_name_override

    dataset = ctx.authenticated_client.dataset_read(organization_name, dataset_name)
    if src_dataset_version_name != 'latest':
        try:
            ctx.authenticated_client.dataset_version_read(dataset.volume_id, src_dataset_version_name)
        except NotFoundAPIException:
            raise ExitException(f'Invalid dataset dataset_version: {dataset_full_name}\n'
                                f'Please check your dataset and dataset_version exist in organization `{organization_name}`.')
    volume_file_copy(
        ctx,
        source_volume_id=dataset.volume_id,
        source_dataset_version=src_dataset_version_name,
        source_path=source_path,
        dest_volume_id=dataset.volume_id,
        dest_dataset_version='latest',
        dest_path=dest_path,
        recursive=recursive,
        watch=watch,
    )



import subprocess

import typer
from terminaltables import AsciiTable

from savvihub.cli.exceptions import ExitException
from savvihub.cli.inputs.organization import organization_name_option
from savvihub.cli.inputs.project import version_control_project_name_option
from savvihub.cli.typer import Typer, Context

project_app = Typer()


@project_app.callback()
def main():
    """
    View projects and pull source codes
    """


@project_app.command(user_required=True)
def list(
    ctx: Context,
    organization_name: str = organization_name_option,
):
    """
    Display a list of projects
    """
    projects_response = ctx.authenticated_client.project_list(organization_name)
    table = AsciiTable([
        ['NAME', 'TYPE', 'REPO'],
        *[[p.name, p.type, p.cached_git_http_url_to_repo or 'N/A']
          for p in projects_response.results],
    ])
    table.inner_column_border = False
    table.inner_heading_row_border = False
    table.inner_footing_row_border = False
    table.outer_border = False

    typer.echo(table.table)


@project_app.command(user_required=True)
def pull(
    ctx: Context,
    organization_name: str = organization_name_option,
    project_name: str = version_control_project_name_option,
):
    """
    Download the project source code
    """
    project = ctx.authenticated_client.project_read(organization_name, project_name)
    if project.type != 'version-control':
        raise ExitException('Please select a version-control project')

    github_token_response = ctx.authenticated_client.github_token()

    dirname = project.cached_git_repo_slug
    try:
        subprocess.check_output(['git', 'clone', f'https://{github_token_response.token}@github.com/{project.cached_git_owner_slug}/{project.cached_git_repo_slug}.git'])
    except subprocess.CalledProcessError:
        dirname = f'savvihub-{project.cached_git_repo_slug}'
        typer.echo(f'Falling back to \'{dirname}\'...')
        subprocess.check_output(['git', 'clone', f'https://{github_token_response.token}@github.com/{project.cached_git_owner_slug}/{project.cached_git_repo_slug}.git', dirname])

    subprocess.check_output(['rm', '-rf', f'{dirname}/.git'])
